@@ -5,54 +5,56 @@
 $::PROTOCOL = '(?:s?https?|ftp)';
 $::RXURL    = '(?:s?https?|ftp)://[-\\w.!~*\'();/?:@&=+$,%#]+' ;
 $::charset  = 'EUC-JP';
-$::version  = '1.1.6_1';
+$::version  = '1.1.7_0';
 %::form     = ();
 $::me       = $::postme = $ENV{SCRIPT_NAME};
 $::print    = ' 'x 10000; $::print = '';
 %::config   = ( crypt => '' , sitename => 'wifky!' );
 
-binmode(STDOUT);
-binmode(STDIN);
+if( $0 eq __FILE__ ){
+    binmode(STDOUT);
+    binmode(STDIN);
 
-eval{
-    local $SIG{ALRM} = sub { die("Time out"); };
-    eval{ alarm 60; };
+    eval{
+        local $SIG{ALRM} = sub { die("Time out"); };
+        eval{ alarm 60; };
 
-    &read_form(\%::form);
-    &change_directory;
-    foreach my $pl (sort grep(/\.plg$/,&directory) ){
-        do $pl; $@ and die($@);
-    }
-    &load_config;
-    &init_globals;
-    foreach my $pl (sort grep(/\.pl$/,&directory) ){
-        do $pl; $@ and die($@);
-    }
-
-    if( exists $::form{a} && exists $::action_plugin{$::form{a}} ){
-        $::action_plugin{ $::form{a} }->();
-    }elsif( exists $::form{p} ){ # page view
-        if( exists $::form{f} ){ # output attachment
-            &action_cat;
-        }else{ # output page itself.
-            &action_view($::form{p});
+        &read_form;
+        &change_directory;
+        foreach my $pl (sort grep(/\.plg$/,&directory) ){
+            do $pl; $@ and die($@);
         }
-    }elsif( &object_exists($::config{FrontPage}) ){
-        &action_view($::config{FrontPage});
-    }else{
-        &do_index('recent','rindex','-l');
+        &load_config;
+        &init_globals;
+        foreach my $pl (sort grep(/\.pl$/,&directory) ){
+            do $pl; $@ and die($@);
+        }
+
+        if( exists $::form{a} && exists $::action_plugin{$::form{a}} ){
+            $::action_plugin{ $::form{a} }->();
+        }elsif( exists $::form{p} ){ # page view
+            if( exists $::form{f} ){ # output attachment
+                &action_cat;
+            }else{ # output page itself.
+                &action_view($::form{p});
+            }
+        }elsif( &object_exists($::config{FrontPage}) ){
+            &action_view($::config{FrontPage});
+        }else{
+            &do_index('recent','rindex','-l');
+        }
+        &flush;
+        eval{ alarm 0; };
+    };
+    if( $@ ){
+        print "Content-Type: text/html;\n\n<html><body>\n",
+              &errmsg($@),"\n</body></html>\n";
     }
-    &flush;
-    eval{ alarm 0; };
-};
-if( $@ ){
-    print "Content-Type: text/html;\n\n<html><body>\n",
-          &errmsg($@),"\n</body></html>\n";
+    exit(0);
 }
-exit(0);
 
 sub change_directory{
-    my $pagedir = $0 ; $pagedir =~ s/\.\w+$/.dat/;
+    my $pagedir = __FILE__ ; $pagedir =~ s/\.\w+$/.dat/;
     unless( chdir $pagedir ){
         mkdir($pagedir,0755);
         chdir $pagedir or die("can not access $pagedir.");
@@ -158,11 +160,9 @@ sub init_globals{
         qq(Generated by <a href="http://wifky.sourceforge.jp">wifky</a> $::version with Perl $])
     );
 
-    @::formtext = ();
-
     %::preferences = (
         ' General Options' => [
-            { desc=>'script-revision '.$::version.' $Date: 2006/12/04 15:27:28 $' ,
+            { desc=>'script-revision '.$::version.' $Date: 2007/03/06 15:34:36 $' ,
               type=>'rem' },
             { desc=>'The sitename', name=>'sitename', size=>40 },
             { desc=>'Enable link to file://...', name=>'locallink' ,
@@ -182,7 +182,6 @@ sub init_globals{
             { desc=>'Subsubsection mark' , name=>'subsubsectionmark' , size=>3 }
         ]
     );
-
     %::inline_syntax_plugin = (
         '100_innerlink1' => \&preprocess_innerlink1 ,
         '200_innerlink2' => \&preprocess_innerlink2 ,
@@ -194,7 +193,6 @@ sub init_globals{
         '800_plugin'     => \&preprocess_plugin ,
         '900_rawurl'     => \&preprocess_rawurl ,
     );
-
     %::block_syntax_plugin = (
         '100_list'       => \&block_listing   ,
         '200_definition' => \&block_definition ,
@@ -207,14 +205,29 @@ sub init_globals{
         '900_seperator'  => \&block_separator ,
         '990_normal'     => \&block_normal ,
     );
-    @::footer_plugin    = ( \&plugin_footnote_flush );
-    @::atodekaku_plugin = ( \&atodekaku_outline , \&unverb );
+    %::call_syntax_plugin = (
+        '100_verbatim'       => \&call_verbatim ,
+        '500_block_syntax'   => \&call_block ,
+        '800_close_sections' => \&call_close_sections ,
+        '900_footer'         => \&call_footnote ,
+    );
+    %::final_plugin = (
+        '500_outline'  => \&final_outline ,
+        '900_verbatim' => \&unverb ,
+    );
+    %::form_list = (
+        '100_textarea'       => \&form_textarea ,
+        '200_preview_botton' => \&form_preview_button ,
+        '300_signarea'       => \&form_signarea ,
+        '400_submit'         => \&form_commit_button ,
+        '500_attachemnt'     => \&form_attachment ,
+    );
 
     @::outline = ();
 }
 
 sub read_multimedia{
-    my ($query_string , $cutter , $form ) = @_;
+    my ($query_string , $cutter ) = @_;
 
     my @blocks = split("\r\n${cutter}","\r\n$query_string");
     foreach my $block (@blocks){
@@ -230,34 +243,36 @@ sub read_multimedia{
         }
         ($header =~ /^Content-Type: image\/(\w+)/ ) and $type = $1;
 
-        $form->{ $name } = $body;
-        length($type) > 0 and $form->{ "${name}.type" } = $type;
-        $form->{ "${name}.filename" } = $fname;
+        $::form{ $name } = $body;
+        length($type) > 0 and $::form{ "${name}.type" } = $type;
+        $::form{ "${name}.filename" } = $fname;
     }
 }
 
-sub read_form{
-    my $form=shift;
-    my $query_string;
-    if( exists $ENV{REQUEST_METHOD} && $ENV{REQUEST_METHOD} eq 'POST' ){
-        $ENV{CONTENT_LENGTH} > 1024*1024 and die('Too large form data');
-        read(STDIN, $query_string, $ENV{CONTENT_LENGTH});
-        if( $query_string =~ /\A(-------.*?)\r?\n/ ){
-            &read_multimedia($query_string,$1,$form);
-            return;
-        }
-    } else {
-        $query_string = (exists $ENV{QUERY_STRING} ? $ENV{QUERY_STRING} : '');
-    }
-    foreach my $p ( split(/[&;]/, $query_string) ){
+sub read_simpleform{
+    foreach my $p ( split(/[&;]/, $_[0]) ){
         my ($name, $value) = split(/=/, $p,2);
         defined($value) or $value = '' ;
         $value =~ s/\+/ /g;
         $value =~ s/%([0-9a-fA-F][0-9a-fA-F])/pack('C', hex($1))/eg;
-        $form->{$name} = $value;
+        $::form{$name} = $value;
     }
 }
 
+sub read_form{
+    if( exists $ENV{REQUEST_METHOD} && $ENV{REQUEST_METHOD} eq 'POST' ){
+        $ENV{CONTENT_LENGTH} > 1024*1024 and die('Too large form data');
+        my $query_string;
+        read(STDIN, $query_string, $ENV{CONTENT_LENGTH});
+        if( $query_string =~ /\A(--.*?)\r?\n/ ){
+            &read_multimedia( $query_string , $1 );
+        }else{
+            &read_simpleform( $query_string );
+        }
+    }
+    &read_simpleform( $ENV{QUERY_STRING} ) if exists $ENV{QUERY_STRING};
+}
+
 sub puts{
     grep(($::print .= "$_\r\n",0),@_);
 }
@@ -269,7 +284,7 @@ sub putenc{
 }
 
 sub flush{
-    grep( $_->( \$main::print ) , @::atodekaku_plugin );
+    grep( $::final_plugin{$_}->( \$::print ) , sort keys %::final_plugin );
     print $::print;
 }
 
@@ -379,54 +394,66 @@ sub title2url{ &myurl( { p=>$_[0] } ); }
 sub attach2url{ &myurl( { p=>$_[0] , f=>$_[1]} );}
 sub is{ $::config{$_[0]} && $::config{$_[0]} ne 'NG' ; }
 
-sub print_form{
-    my ($title,$html,$stamp) = @_;
+sub form_textarea{
+    &putenc('<textarea cols="80" rows="20" name="honbun">%s</textarea><br>'
+            , ${$_[0]} );
+}
+sub form_preview_button{
+    &puts('<input type="submit" name="a" value="Preview">');
+}
+sub form_signarea{
+    exists $::form{admin} or &is_frozen() or return;
 
-    &putenc('<div class="update"><form name="editform" action="%s"
-          enctype="multipart/form-data" method="post"
-          accept-charset="%s" ><input type="hidden" name="stamp" value="%s"
-        ><input type="hidden" name="p" value="%s"><br>'
-        , $::postme , $::charset , $stamp , $title );
-    &puts('<textarea cols="80" rows="20" name="honbun">'.$html.
-            '</textarea><br>');
+    &puts('<input type="hidden" name="admin" value="admin">');
+
+    &print_signarea();
+
+    &puts('<input type="checkbox" name="to_freeze" value="1"');
+    &is_frozen() and &puts('checked');
+    &puts(' >freeze');
 
     my $fname=&title2fname( $::form{p} );
-    &puts('<input type="submit" name="a" value="Preview">');
-    if( exists $::form{admin} || &is_frozen() ){
-        &print_signarea();
-        &puts('<input type="checkbox" name="to_freeze" value="1"');
-        &is_frozen() and &puts('checked');
-        &puts(' >freeze <input type="hidden" name="admin" value="admin">');
-        if( &mtimeraw($fname) ){
-            &puts('<input type="checkbox" name="sage" value="1">sage');
-        }
+    if( &mtimeraw($fname) ){
+        &puts('<input type="checkbox" name="sage" value="1">sage');
     }
+}
+sub form_commit_button{
     &puts('<input type="submit" name="a" value="Commit">');
+}
 
+sub form_attachment{
     &puts('<h2>Attachment</h2>
     <p>New:<input type="file" name="butsu" size="48">');
     if( exists $::form{admin} || &is_frozen() ){
         &print_signarea('qassword');
     }
     &puts('<input type="submit" name="a" value="Upload"></p>');
+    my @attachments=&list_attachment( $::form{p} ) or return;
+    &puts('<p>');
+    foreach my $attach (sort @attachments){
+        &putenc('<input type="radio" name="f" value="%s" ><input
+                type="text" name="dummy" readonly value="&lt;&lt;{%s}"
+                size="20" style="font-family:monospace"
+                onClick="this.select();">'
+              ,$attach ,$attach );
 
-    if( my @attachments=&list_attachment( $::form{p} ) ){
-        &puts('<p>');
-        foreach my $attach (sort @attachments){
-            &putenc('<input type="radio" name="f" value="%s" ><input
-                    type="text" name="dummy" readonly value="&lt;&lt;{%s}"
-                    size="20" style="font-family:monospace"
-                    onClick="this.select();">'
-                  ,$attach ,$attach );
-
-            my $fn = &title2fname($::form{p}, $attach);
-            &putenc('(%d bytes, at %s)',(stat $fn)[7],&mtime($fn));
-            &puts('<br>');
-        }
-        &puts('</p><input type="submit" name="a" value="Delete">
-            <input type="submit" name="dummybotton" value="Download">');
+        my $fn = &title2fname($::form{p}, $attach);
+        &putenc('(%d bytes, at %s)',(stat $fn)[7],&mtime($fn));
+        &puts('<br>');
     }
-    &puts(@::formtext);
+    &puts('</p><input type="submit" name="a" value="Delete">
+        <input type="submit" name="dummybotton" value="Download">');
+}
+
+sub print_form{
+    my ($title,$newsrc,$orgsrc) = @_;
+
+    &putenc('<div class="update"><form name="editform" action="%s"
+          enctype="multipart/form-data" method="post"
+          accept-charset="%s" ><input type="hidden" name="orgsrc" value="%s"
+        ><input type="hidden" name="p" value="%s"><br>'
+        , $::postme , $::charset , &yen($$orgsrc) , $title );
+    grep( $::form_list{$_}->($newsrc), sort keys %::form_list );
     &puts('</form></div>');
 }
 
@@ -464,8 +491,7 @@ sub print_header{
         }
     }
     &puts('--></style></head>');
-    &puts( exists $::form{p} && &is_frozen($::form{p})
-                ? '<body class="frozen">' : '<body>' );
+    &puts( &is_frozen() ? '<body class="frozen">' : '<body>' );
     &puts( @::body_header );
     &putenc('<div class="%s">' , $divclass );
 
@@ -492,7 +518,10 @@ sub print_copyright{
 }
 
 sub is_frozen{
-    if( -r &title2fname(exists $::form{p} ? $::form{p} : $::config{FrontPage})){
+    if( -r &title2fname(  $#_>=0            ? $_[0]
+                        : exists $::form{p} ? $::form{p}
+                        : $::config{FrontPage}))
+    {
         ! -w _;
     }else{
         &is('lonely');
@@ -519,10 +548,10 @@ sub check_frozen{
     }
 }
 sub check_conflict{
-    my $stamp_time = &mtimeraw(&title2fname($::form{p}));
-    if( $stamp_time > 0 && (my $begin_time = $::form{stamp}) ne $stamp_time ){
-        die( "!Someone else modified this page on ${stamp_time}".
-             " after you began to edit on ${begin_time}!"  );
+    my $current_source = &read_object($::form{p});
+    my $before_source  = &deyen($::form{orgsrc});
+    if( $current_source ne $before_source ){
+        die( "!Someone else modified this page after you began to edit."  );
     }
 }
 
@@ -600,17 +629,14 @@ sub action_query_delete{
             , $::form{f} , $::form{p} );
     &is_frozen() and &print_signarea();
     &putenc('<div>Are you sure ? </div>
-        <input type="submit" name="yes" value="Yes">
-        <input type="submit" name="no" value="No">
-        <input type="hidden" name="a" value="del">
-        <input type="hidden" name="p" value="%s">
-        <input type="hidden" name="f" value="%s">
-        <input type="hidden" name="stamp" value="%s">
-        <input type="hidden" name="encrlf" value="YES">'
-            , $::form{p} , $::form{f} , $::form{stamp} );
-    &putenc( '<input type="hidden" name="honbun" value="%s"></p></form>',
-                &yen($::form{honbun}) );
-
+        <input type="submit" name="yes"    value="Yes">
+        <input type="submit" name="no"     value="No">
+        <input type="hidden" name="a"      value="del">
+        <input type="hidden" name="p"      value="%s">
+        <input type="hidden" name="f"      value="%s">
+        <input type="hidden" name="orgsrc" value="%s">
+        <input type="hidden" name="yensrc" value="%s"></p></form>',
+            , $::form{p} , $::form{f} , $::form{orgsrc} , &yen($::form{honbun}) );
     &print_footer;
 }
 
@@ -857,14 +883,15 @@ sub action_upload{
 sub do_submit{
     my $title=$::form{p};
     my $fn=&title2fname($title);
+    my $sagetime=&mtimeraw($fn);
 
     &is_frozen() and chmod(0644,$fn);
 
     defined($::hook_submit) and $::hook_submit->(\$title , \$::form{honbun});
-    if( &write_object( $title , \$::form{honbun} ) ){
-        $::form{to_freeze} and chmod 0444,$fn;
-        $::form{sage} and utime($::form{stamp},$::form{stamp},$fn);
-        &transfer_page;
+    if( &write_file( $fn , \$::form{honbun} ) ){
+        chmod 0444,$fn if $::form{to_freeze};
+        utime($sagetime,$sagetime,$fn) if $::form{sage};
+        &transfer_page();
     }else{
         &transfer_url($::me);
     }
@@ -887,15 +914,15 @@ sub transfer_page{
 sub do_preview{
     my $e_message = shift;
     my $title = $::form{p};
-    my $html  = &enc($::form{honbun});
-    exists $::form{encrlf} && $::form{encrlf} eq 'YES' and $html=&deyen($html);
+    $::form{honbun} = &deyen($::form{yensrc}) if exists $::form{yensrc};
+    $::form{orgsrc} = &deyen($::form{orgsrc} ||'');
 
     &print_header(divclass=>'max',title=>'Edit');
     defined($e_message) and &puts(qq(<div class="warning">${e_message}</div>));
     &begin_day("Preview: $title");
-        &print_page( title=>$title , html=>$html , index=>1 , main=>1 );
+        &print_page( title=>$title , source=>\$::form{honbun} , index=>1 , main=>1 );
     &end_day();
-    &print_form( $title , $html , $::form{stamp} );
+    &print_form( $title , \$::form{honbun} , \$::form{orgsrc} );
     &print_footer;
 }
 
@@ -904,7 +931,8 @@ sub action_edit{
     &print_header(divclass=>'max',title=>'Edit');
     &begin_day("Edit: $title");
     my $fn=&title2fname($title);
-    &print_form( $title , &enc(&read_file($fn)), &mtimeraw($fn) );
+    my $source=&read_file($fn);
+    &print_form( $title , \$source , \$source );
 
     if( &object_exists($::form{p}) && exists $::form{admin} ){
         &putenc('<h2>Rename</h2>
@@ -990,7 +1018,7 @@ sub list_attachment{
 sub print_page{
     my %args=( @_ );
     my $title=$args{title};
-    my $html =( exists $args{html} ? $args{html} : &enc( &read_object($title)) );
+    my $html =&enc( exists $args{source} ? ${$args{source}} : &read_object($title));
     return 0 unless $html;
 
     push(@main::outline,
@@ -1040,13 +1068,13 @@ sub strip_tag{
     $text;
 }
 
-sub verbatim{
-    my $html=shift;
-    $$html =~ s!^\s*\&lt;pre&gt;(.*?\n)\s*\&lt;/pre&gt;|^\s*8\&lt;(.*?\n)\s*\&gt;8|`(.)`(.*?)`\3`!
-                  defined($4)
-                ? &verb('<tt class="pre">'.&cr2br($4).'</tt>')
-                : "\n\n<pre>".&verb(defined($1) ? $1 : $2)."</pre>\n\n"
-            !gesm;
+sub call_verbatim{
+    ${$_[0]} =~
+    s!^\s*\&lt;pre&gt;(.*?\n)\s*\&lt;/pre&gt;|^\s*8\&lt;(.*?\n)\s*\&gt;8|`(.)`(.*?)`\3`!
+    defined($4)
+    ? &verb('<tt class="pre">'.&cr2br($4).'</tt>')
+    : "\n\n<pre>".&verb(defined($1) ? $1 : $2)."</pre>\n\n"
+    !gesm;
 }
 
 sub inner_link{
@@ -1097,8 +1125,8 @@ sub plugin_footnote{
     '</sup>' ;
 }
 
-sub plugin_footnote_flush{
-    my $session = shift;
+sub call_footnote{
+    my (undef,$session) = shift;
     my $footnotes = $session->{footnotes};
     return unless $footnotes;
 
@@ -1120,7 +1148,7 @@ sub plugin_outline{
     "\x1B(outline)";
 }
 
-sub atodekaku_outline{
+sub final_outline{
     my ($print)=@_;
     my $depth=-2;
     my $ss='';
@@ -1416,20 +1444,23 @@ sub midashi{
 }
 
 sub syntax_engine{
-    my ($html,$session) = @_;
-    &default_syntax_engine( ref($html) ? $html : \$html , $session );
-    foreach my $proc (@main::footer_plugin){ $proc->( $session ); }
+    my ($ref2html,$session) = ( ref($_[0]) ? $_[0] : \$_[0] , $_[1] );
+    foreach my $p ( sort keys %::call_syntax_plugin ){
+        $::call_syntax_plugin{$p}->( $ref2html , $session );
+    }
 }
 
-sub default_syntax_engine{
-    my ($ref2html,$session) = @_;
-    &verbatim( $ref2html );
-
+sub call_block{
+    my ($ref2html,$session)=@_;
     foreach my $fragment( split(/\r?\n\r?\n/,$$ref2html) ){
         foreach my $p (sort keys %::block_syntax_plugin){
             $::block_syntax_plugin{$p}->($fragment,$session) and last;
         }
     }
+}
+
+sub call_close_sections{
+    my ($ref2html,$session)=@_;
     exists $session->{section} and
         grep( $_ && &puts('</div></div>'),@{$session->{section}} );
 }
